/*****************************************************************************

       Copyright   1995, 1996 Digital Equipment Corporation,
                       Maynard, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, provided  
that the copyright notice and this permission notice appear in all copies  
of software and supporting documentation, and that the name of Digital not  
be used in advertising or publicity pertaining to distribution of the software 
without specific, written prior permission. Digital grants this permission 
provided that you prominently mark, as not part of the original, any 
modifications made to this software or documentation.

Digital Equipment Corporation disclaims all warranties and/or guarantees  
with regard to this software, including all implied warranties of fitness for 
a particular purpose and merchantability, and makes no representations 
regarding the use of, or the results of the use of, the software and 
documentation in terms of correctness, accuracy, reliability, currentness or
otherwise; and you rely on the software, documentation and results solely at 
your own risk. 

******************************************************************************/


/*****************************************************************************
*
* Copyright 1996, 1997, 1998 Hans Reiser, see /usr/src/linux/reiserfs/README
*                     for licensing and copyright details
*
*****************************************************************************/



#ifndef REISERFS_LIB_H
#define REISERFS_LIB_H

/*
 * Special inodes numbers
 */
#if 0
#define	REISEFS_BAD_INO		 1	/* Bad blocks inode */
#define REISEFS_ROOT_INO		 2	/* Root inode */
#define REISEFS_ACL_IDX_INO	 3	/* ACL inode */
#define REISEFS_ACL_DATA_INO	 4	/* ACL inode */
#define REISEFS_BOOT_LOADER_INO	 5	/* Boot loader inode */
#define REISEFS_UNDEL_DIR_INO	 6	/* Undelete directory inode */
#define REISEFS_FIRST_INO		11	/* First non reserved inode */

/*
 * Maximal count of links to a file
 */
#define REISEFS_LINK_MAX		32000

/*
 * Macro-instructions used to manage several block sizes
 */
#define REISEFS_MIN_BLOCK_SIZE		1024
#define	REISEFS_MAX_BLOCK_SIZE		4096
#define REISEFS_MIN_BLOCK_LOG_SIZE		  10
#define REISEFS_BLOCK_SIZE(s)		(REISEFS_MIN_BLOCK_SIZE << (s)->s_log_block_size)
#define REISEFS_ACLE_PER_BLOCK(s)		(REISEFS_BLOCK_SIZE(s) / sizeof (struct reisefs_acl_entry))
#define	REISEFS_ADDR_PER_BLOCK(s)		(REISEFS_BLOCK_SIZE(s) / sizeof (unsigned int))
#define REISEFS_BLOCK_SIZE_BITS(s)	((s)->s_log_block_size + 10)
#define	REISEFS_INODES_PER_BLOCK(s)	(REISEFS_BLOCK_SIZE(s) / sizeof (struct reisefs_inode))

/*
 * Macro-instructions used to manage fragments
 */
#define REISEFS_MIN_FRAG_SIZE		1024
#define	REISEFS_MAX_FRAG_SIZE		4096
#define REISEFS_MIN_FRAG_LOG_SIZE		  10
#define REISEFS_FRAG_SIZE(s)		(REISEFS_MIN_FRAG_SIZE << (s)->s_log_frag_size)
#define REISEFS_FRAGS_PER_BLOCK(s)	        (REISEFS_BLOCK_SIZE(s) / REISEFS_FRAG_SIZE(s))

//#define	REISERFS_NDIR_BLOCKS		12
//#define	REISERFS_IND_BLOCK			REISERFS_NDIR_BLOCKS
//#define	REISERFS_DIND_BLOCK			(REISERFS_IND_BLOCK + 1)
//#define	REISERFS_TIND_BLOCK			(REISERFS_DIND_BLOCK + 1)
//#define	REISERFS_N_BLOCKS			(REISERFS_TIND_BLOCK + 1)

/*
 * Inode flags
 */
#define	REISERFS_SECRM_FL			0x0001	/* Secure deletion */
#define	REISERFS_UNRM_FL			0x0002	/* Undelete */
#define	REISERFS_COMPR_FL			0x0004	/* Compress file */
#define REISERFS_SYNC_FL			0x0008	/* Synchronous updates */


#endif

/*
 * Structure of an inode on the disk
 */
struct reiserfs_inode {

	__u32 i_key [4];
	__u32 i_first_direct_byte;
	int i_data_length;
	__u32 i_data [REISERFS_N_BLOCKS];

	unsigned short i_mode;	/* File mode */
	unsigned short i_uid;	/* Owner Uid */
	unsigned int i_size;	/* Size in bytes */
	unsigned int i_atime;	/* Access time */
	unsigned int i_ctime;	/* Creation time */
	unsigned int i_mtime;	/* Modification time */
	unsigned int i_dtime;	/* Deletion Time */
	unsigned short i_gid;	/* Group Id */
	unsigned short i_links_count;	/* Links count */
	unsigned int i_blocks;	/* Blocks count */
	unsigned int i_flags;	/* File flags */
/*	unsigned int i_reserved1;*/
	unsigned int i_block[REISERFS_N_BLOCKS];	/* Pointers to blocks */
	unsigned int i_version;	/* File version (for NFS) */
	unsigned int i_file_acl;	/* File ACL */
	unsigned int i_dir_acl;	/* Directory ACL */
	unsigned short i_pad1;
/*	unsigned int i_reserved2[2];*/
};

/*
 * Mount flags
 */
#define REISERFS_MOUNT_CHECK_NORMAL		0x0001	/* Do some more checks */
#define REISERFS_MOUNT_CHECK_STRICT		0x0002	/* Do again more checks */
#define REISERFS_MOUNT_CHECK		(REISERFS_MOUNT_CHECK_NORMAL | \
					 REISERFS_MOUNT_CHECK_STRICT)
#define REISERFS_MOUNT_GRPID		0x0004	/* Create files with directory's group */
#define REISERFS_MOUNT_DEBUG		0x0008	/* Some debugging messages */
#define REISERFS_MOUNT_ERRORS_CONT		0x0010	/* Continue on errors */
#define REISERFS_MOUNT_ERRORS_RO		0x0020	/* Remount fs ro on errors */
#define REISERFS_MOUNT_ERRORS_PANIC		0x0040	/* Panic on errors */

/*
 * Maximal mount counts between two filesystem checks
 */
#define REISERFS_DFL_MAX_MNT_COUNT		20	/* Allow 20 mounts */
#define REISERFS_DFL_CHECKINTERVAL		0	/* Don't use interval check */

/*
 * Behaviour when detecting errors
 */
#define REISERFS_ERRORS_CONTINUE		1	/* Continue execution */
#define REISERFS_ERRORS_RO			2	/* Remount fs read-only */
#define REISERFS_ERRORS_PANIC		3	/* Panic */
#define REISERFS_ERRORS_DEFAULT		REISERFS_ERRORS_CONTINUE

#endif				/* REISERFS_LIB_H */
